"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const router_1 = require("next/router");
const react_1 = require("react");
const query_string_1 = __importDefault(require("query-string"));
function useQuery(types) {
    const router = (0, router_1.useRouter)();
    const serialized = serializeTypes(types !== null && types !== void 0 ? types : {});
    return (0, react_1.useMemo)(() => {
        const deserialized = deserializeTypes(serialized);
        const [, query] = router.asPath.split('?');
        const parsed = query_string_1.default.parse(query, {
            arrayFormat: "comma"
        });
        const entries = Object.entries(deserialized);
        if (!entries.length) {
            return parsed;
        }
        return entries.reduce((query, [key, type]) => {
            const value = parsed[key];
            const result = (() => {
                if (!Array.isArray(type)) {
                    return type(value);
                }
                if (Array.isArray(value) && Array.isArray(type)) {
                    return value.map(v => type[0](v));
                }
                return value;
            })();
            return (Object.assign(Object.assign({}, query), { [key]: result }));
        }, {});
    }, [router.asPath, serialized]);
}
function serializeTypes(types) {
    const serialized = Object.entries(types).reduce((serialized, [key, type]) => {
        const typeConstructor = Array.isArray(type) ? type[0] : type;
        const typeString = (() => {
            switch (typeConstructor) {
                case Number:
                    return 'Number';
                case String:
                    return 'String';
                case Boolean:
                    return 'Boolean';
            }
            return 'Unknown';
        })();
        return Object.assign(Object.assign({}, serialized), { [key]: Array.isArray(type) ? `[]${typeString}` : typeString });
    }, {});
    return JSON.stringify(serialized);
}
function deserializeTypes(serialized) {
    const json = JSON.parse(serialized);
    const deserialized = Object.entries(json).reduce((deserialized, [key, type]) => {
        const isArray = type.startsWith('[]');
        const typeString = isArray ? type.substring(2) : type;
        const typeConstructor = (() => {
            switch (typeString) {
                case 'Number':
                    return Number;
                case 'String':
                    return String;
                case 'Boolean':
                    return Boolean;
            }
            return undefined;
        })();
        return Object.assign(Object.assign({}, deserialized), { [key]: isArray ? [typeConstructor] : typeConstructor });
    }, {});
    return deserialized;
}
exports.default = useQuery;
